@@ -1,16 +1,12 @@
 package Pod::Autopod; ## Generates pod documentation by analysing perl modules.
-
-
+$Pod::Autopod::VERSION = '1.202';
+use 5.006; #Pod::Abstract uses features of 5.6
 use FileHandle;
 use strict;
 use Pod::Abstract;
 use Pod::Abstract::BuildNode qw(node nodes);
 
 
-
-use vars qw($VERSION);
-$VERSION = '1.08';
-
 # This Module is designed to generate pod documentation of a perl class by analysing its code.
 # The idea is to have something similar like javadoc. So it uses also comments written directly
 # obove the method definitions. It is designed to asumes a pm file which represents a class.
@@ -53,7 +49,7 @@ $VERSION = '1.08';
 #
 #  # This method is doing foo.
 #  #
-#  #  print $this->foo();
+#  #  print $self->foo();
 #  #
 #  # 
 #  # It is not doing bar, only foo.
@@ -75,9 +71,9 @@ $VERSION = '1.08';
 #
 # The example above produces the following method description: 
 #
-#  $this->foo($text);
+#  $self->foo($text);
 #
-# The object "$this" is the default and automatially used when a constructor was found ("new")
+# The object "$self" is the default and automatially used when a constructor was found ("new")
 # or the class inherits with ISA or "use base".
 # You can change this by the parameter "selfstring" in the autopod constructor.
 #
@@ -95,7 +91,7 @@ $VERSION = '1.08';
 #
 # Produces:
 #
-#  $this->foo($scalar);
+#  $self->foo($scalar);
 #
 # As you see, that was the quickest way to write the definition. The keywork "void" is default.
 #
@@ -118,7 +114,7 @@ $VERSION = '1.08';
 #
 # produces:
 #
-#  my $state = $this->foo($firstname, $lastname, \%persondata);
+#  my $state = $self->foo($firstname, $lastname, \%persondata);
 #
 # or write it in java style:
 #
@@ -134,7 +130,7 @@ $VERSION = '1.08';
 #
 # produces:
 #
-#  my ($a, $b) = $this->foo($text);
+#  my ($a, $b) = $self->foo($text);
 #
 #
 # If you want to use key values pairs as in a hash, you may describe it like:
@@ -153,7 +149,7 @@ $VERSION = '1.08';
 #
 # procudes:
 #
-#   my $lista | \$refb = $this->foo(\@list | $text, $flag);
+#   my $lista | \$refb = $self->foo(\@list | $text, $flag);
 #
 # Of course, that is not an official perl syntax with the or "|", but it shows
 # you that is expected.
@@ -184,7 +180,7 @@ $VERSION = '1.08';
 #
 # The keyvalues are not mandatory.
 #
-# selfstring may hold something like '$self' as alternative to '$this', which is default.
+# selfstring may hold something like '$self' as alternative to '$self', which is default.
 #
 # alsohiddenmethods gets a boolean flag to show also methods which starts with "_".
 #
@@ -193,61 +189,61 @@ my $pkg=shift;
 my %v=@_; 
 
 
-	my $this={};
-	bless $this,$pkg;
+	my $self={};
+	bless $self,$pkg;
 
-	$this->{package}=$pkg;
+	$self->{package}=$pkg;
 
 	foreach my $k (keys %v){ ## sets values to object
-		$this->{$k}=$v{$k};
+		$self->{$k}=$v{$k};
 	}  
 	
-	$this->{'selfstring'} = $this->{'selfstring'} || '$this';
+	$self->{'selfstring'} = $self->{'selfstring'} || '$self';
 	
 
-	if ($this->{'readfile'}){
-		$this->readFile($this->{'readfile'});
+	if ($self->{'readfile'}){
+		$self->readFile($self->{'readfile'});
 	}
 
 
-	if ($this->{'writefile'}){
-		$this->writeFile($this->{'writefile'});
+	if ($self->{'writefile'}){
+		$self->writeFile($self->{'writefile'});
 	}
 
 
-	if ($this->{'readdir'}){
-		$this->readDirectory($this->{'readdir'});
+	if ($self->{'readdir'}){
+		$self->readDirectory($self->{'readdir'});
 	}	
 
-return $this;
+return $self;
 }  
 
 
 ## Returns the border string which delimit the perl code and pod inside a pm file.
 sub getBorderString{ ## $scalar
-my $this=shift;
-my $pkg=$this->{'package'};
+my $self=shift;
+my $pkg=$self->{'package'};
 
-	if ($this->{'BORDER'} eq ''){
+	if ($self->{'BORDER'} eq ''){
 		
 		my $border = '#' x 20;
 		$border .= " pod generated by $pkg - keep this line to make pod updates possible ";
 		$border .= '#' x 20;
-		$this->{'BORDER'}=$border;
+		$self->{'BORDER'}=$border;
 		
 	}
 
-return $this->{'BORDER'};	
+return $self->{'BORDER'};	
 }
 
 
 ## Set an alternative border string. 
 ## If you change this, you have to do it again when updating the pod.
 sub setBorderString{ ## void ($borderstring)
-my $this=shift;
+my $self=shift;
 my $s=shift;
 
-	$this->{'BORDER'} =$s;
+	$self->{'BORDER'} =$s;
 
 }
 
@@ -256,10 +252,13 @@ my $s=shift;
 # Expects Perl code as arrayref
 # or text (scalar).
 # 
-# When used, it automatically runs scanArray(). 
-sub setPerlCode{ ## void ($text|\@array)
-my $this=shift;
+# When used, it automatically runs scanArray().
+# This now passes the filename to be used in case
+# we are podding a .pl or .cgi file. NW 
+sub setPerlCode{ ## void ($text|\@array, $file)
+my $self=shift;
 my $code=shift;
+my $file=shift;
 
 	my $arr; 
 
@@ -270,20 +269,20 @@ my $code=shift;
 		$arr=$code;
 	}
 
-	$this->{'PERL_CODE'}=$arr;
+	$self->{'PERL_CODE'}=$arr;
 
-	$this->scanArray($arr);	
-	$this->buildPod();
+	$self->scanArray($arr, $file);	
+	$self->buildPod();
 }
 
 
 # Returns perl code which was set before.
 sub getPerlCode{# $text
-my $this=shift;
+my $self=shift;
 	
-	my $border = $this->getBorderString();
+	my $border = $self->getBorderString();
 	
-	my $arr = $this->{'PERL_CODE'};
+	my $arr = $self->{'PERL_CODE'};
 	
 	my @code;
 	foreach my $row (@$arr){
@@ -302,15 +301,15 @@ return $text;
 
 # Returns the pod formated text.s
 sub getPod{ ## $text
-my $this=shift;
+my $self=shift;
 
-return $this->{"POD_TEXT"};	
+return $self->{"POD_TEXT"};	
 }
 
 
 
 sub _getFileArray{
-my $this=shift;
+my $self=shift;
 my $filename=shift;
 my @f;
 
@@ -327,10 +326,10 @@ return wantarray ? @f : \@f;
 
 
 sub _getFileScalar{
-my $this=shift;
+my $self=shift;
 my $filename=shift;
 	
-	my $a = $this->_getFileArray($filename);
+	my $a = $self->_getFileArray($filename);
 
 return join("",@$a);	
 }
@@ -339,19 +338,19 @@ return join("",@$a);
 
 # writes a pod file
 #
-# If the file has a pm extension, it writes the perl code and the pod
+# If the file has a pm or pl or cgi extension, it writes the perl code and the pod
 # If the file has a pod extension or any, it only writes the pod.
 sub writeFile{ # void ($filename)
-my $this=shift;
+my $self=shift;
 my $file=shift;
-my $pod=$this->getPod();
+my $pod=$self->getPod();
 
-	if ($file=~ m/\.pm$/i){ ## target is pm file, so add perl-code 
-		my $text=$this->getPerlCode();
-		$text.="\n".$this->{'BORDER'}."\n\n$pod";
-		$this->_putFile($file,$text);
+	if ($file=~ m/\.pm$|\.pl$|\.cgi$/i){ ## target is pm or pl or cgi file, so add perl-code 
+		my $text=$self->getPerlCode();
+		$text.="\n".$self->{'BORDER'}."\n\n$pod";
+		$self->_putFile($file,$text);
 	}else{## target is any or pod file, write only pod
-		$this->_putFile($file,$pod);
+		$self->_putFile($file,$pod);
 	}
 	
 }
@@ -359,12 +358,12 @@ my $pod=$this->getPod();
 
 ## Reading a Perl class file and loads it to memory.
 sub readFile{ # void ($filename)
-my $this=shift;
+my $self=shift;
 my $file=shift or die "need filename";
 
 
-	my $arr = $this->_getFileArray($file);
-	$this->setPerlCode($arr);
+	my $arr = $self->_getFileArray($file);
+	$self->setPerlCode($arr, $file);
 	
 	
 }
@@ -384,23 +383,37 @@ my $file=shift or die "need filename";
 ## With verbose it prints the list of written files.
 ##
 sub readDirectory{ # void ($directory,updateonly=>scalar,pod=>scalar,verbose=>scalar)
-my $this=shift;
+my $self=shift;
 my $directory=shift or die "need directory";
 my $v={@_};
 my $updateonly=$v->{'updateonly'};
 my $verbose=$v->{'verbose'};
 my $pod=$v->{'pod'};
-my $border=$this->getBorderString();
+my $poddir=$v->{'poddir'};
+my $border=$self->getBorderString();
 
 
-	my @dir = $this->_getPodFilesRecoursive($directory);
+	my @dir = $self->_getPodFilesRecoursive($directory);
 
 
 	foreach my $filein (@dir){
 		
 		my $fileout = $filein;
+
+    if ($poddir){
+      $pod=1;
+      $fileout=~ s|^$directory|$poddir|;
+
+      my $p=_extractPath($fileout);
+
+
+      if (!-e $p){
+        _makeDirRecursive($p);
+      }
+    }
+
 		
-		my $filecontent = $this->_getFileScalar($filein);
+		my $filecontent = $self->_getFileScalar($filein);
 		if ($updateonly){
 			if (($filecontent!~ m/$border/) &&  ($filecontent!~ m/AUTOPODME/) ){$fileout=undef}; ## no border, no update
 		}
@@ -424,13 +437,13 @@ my $border=$this->getBorderString();
 
 
 sub _getPodFilesRecoursive{
-my $this=shift;
+my $self=shift;
 my $path=shift;
 my %para=@_;
 my @files;
 
-	@files=$this->_getFilesRecoursiveAll($path);
-	$this->_filterFileArray(\@files,ext=>'pm',path=>$path);
+	@files=$self->_getFilesRecoursiveAll($path);
+	$self->_filterFileArray(\@files,ext=>'pm',path=>$path);
 	@files=sort @files;
 
 return wantarray ? @files : \@files;
@@ -438,21 +451,21 @@ return wantarray ? @files : \@files;
 
 
 sub _getFilesRecoursiveAll{
-my $this=shift;
+my $self=shift;
 my $path=shift;
 my %para;
 my @f;
 my @fm;
 
 
-	@f=$this->_getDirArray($path);
+	@f=$self->_getDirArray($path);
 
-	$this->_filterFileArray(\@f);
-	$this->_addPathToArray($path,\@f);
+	$self->_filterFileArray(\@f);
+	$self->_addPathToArray($path,\@f);
 
 	foreach my $d (@f){
 		if (-d $d){
-		push @fm,$this->_getFilesRecoursiveAll($d);
+		push @fm,$self->_getFilesRecoursiveAll($d);
 		}
 	}
 	push @f,@fm;
@@ -465,7 +478,7 @@ return @f;
 
 
 sub _getDirArray{
-my $this=shift;
+my $self=shift;
 my $path=shift;
 my @f;
 my @nf;
@@ -484,7 +497,7 @@ return wantarray ? @nf : \@nf;
 
 
 sub _addPathToArray{
-my $this=shift;
+my $self=shift;
 my $path=shift;
 my $dir_ref=shift;
 
@@ -497,7 +510,7 @@ my $dir_ref=shift;
 
 
 sub _filterFileArray{
-my $this=shift;
+my $self=shift;
 my $dir_ref=shift;
 my %para=@_;
 my @nf;
@@ -533,7 +546,7 @@ my $path=$para{path};
 	
 
 sub _putFile{
-my $this=shift;
+my $self=shift;
 my $file=shift;
 my $text=shift;
 
@@ -549,10 +562,10 @@ my $text=shift;
 # This class may scan the perl code.
 # But it is called automatically when importing a perl code.
 sub scanArray{
-my $this=shift;	
+my $self=shift;	
 my $arr=shift or die "Arrayref expected";	
-	
-	$this->{'STATE'} = 'head';
+my $file=shift;	
+	$self->{'STATE'} = 'head';
 	
 	
 	## reverse read
@@ -561,85 +574,88 @@ my $arr=shift or die "Arrayref expected";
 		
 		my $line = $arr->[$p];
 
-		if ((($line=~ m/^\s*\#/) || ($p == 0)) && ($this->{'STATE'} eq 'headwait')){ ## last line of body
-			$this->{'STATE'} = 'head';
-		}elsif((($line=~ m/^\s*$/) || ($p == 0)) && ($this->{'STATE'} eq 'head')){ ## last line of body
-			$this->{'STATE'} = 'bodywait';
+		if ((($line=~ m/^\s*\#/) || ($p == 0)) && ($self->{'STATE'} eq 'headwait')){ ## last line of body
+			$self->{'STATE'} = 'head';
+		}elsif((($line=~ m/^\s*$/) || ($p == 0)) && ($self->{'STATE'} eq 'head')){ ## last line of body
+			$self->{'STATE'} = 'bodywait';
 		}
 		
-		if (($this->{'STATE'} eq 'headwait') && ($line!~ m/^\s*$/) && ($line!~ m/^\s*\#/)){
-			$this->{'STATE'}='free';
+		if (($self->{'STATE'} eq 'headwait') && ($line!~ m/^\s*$/) && ($line!~ m/^\s*\#/)){
+			$self->{'STATE'}='free';
 		}
 
 
-		if ((($line=~ m/^\s*\}/) || ($p == 0) || ($line=~ m/^\s*sub [^ ]+/)) && ($this->{'STATE'}=~ m/^(head|headwait|bodywait|free)$/)){ ## last line of body
-			$this->_clearBodyBuffer();
-			$this->{'STATE'} = 'body';
-			$this->_addHeadBufferToAttr();
+		if ((($line=~ m/^\s*\}/) || ($p == 0) || ($line=~ m/^\s*sub [^ ]+/)) && ($self->{'STATE'}=~ m/^(head|headwait|bodywait|free)$/)){ ## last line of body
+			$self->_clearBodyBuffer();
+			$self->{'STATE'} = 'body';
+			$self->_addHeadBufferToAttr();
 		}
 
 
 		if ($line=~ m/^\s*sub [^ ]+/){ ## head line
-			$this->_clearHeadBuffer();
-			$this->_setMethodLine($line);
-			$this->{'STATE'} = 'headwait';
-			$this->_addBodyBufferToAttr();
-			$this->_setMethodAttr($this->_getMethodName(),'returnline',$this->_getMethodReturn());
-			$this->_setMethodReturn(undef);	
+			$self->_clearHeadBuffer();
+			$self->_setMethodLine($line);
+			$self->{'STATE'} = 'headwait';
+			$self->_addBodyBufferToAttr();
+			$self->_setMethodAttr($self->_getMethodName(),'returnline',$self->_getMethodReturn());
+			$self->_setMethodReturn(undef);	
 		}
 
-		if ($this->{'STATE'} eq 'head'){
-			$this->_addLineToHeadBuffer($line);
-		}elsif($this->{'STATE'} eq 'body'){
-			$this->_addLineToBodyBuffer($line);	
+		if ($self->{'STATE'} eq 'head'){
+			$self->_addLineToHeadBuffer($line);
+		}elsif($self->{'STATE'} eq 'body'){
+			$self->_addLineToBodyBuffer($line);	
 		}
 		
 		
 		if ($line=~ m/^\s*package ([^\;]+)\;(.*)/){
-			$this->{'PKGNAME'}=$1;
-			$this->{'PKGNAME_DESC'}=$2;
-			$this->{'PKGNAME_DESC'}=~ s/^\s*\#*//g;
+			$self->{'PKGNAME'}=$1;
+			$self->{'PKGNAME_DESC'}=$2;
+			$self->{'PKGNAME_DESC'}=~ s/^\s*\#*//g;
+		} else {
+			$self->{'PKGNAME'}=$file;
+		        $self->{'PKGNAME_DESC'}=$file;
 		}
 
 		if ($line=~ m/^\s*use +([^\; ]+)[\; ](.*)/){
-			$this->{'REQUIRES'} = $this->{'REQUIRES'} || [];
+			$self->{'REQUIRES'} = $self->{'REQUIRES'} || [];
 			my $name=$1;
 			my $rem=$2;
 			$rem=~ s/^[^\#]*\#*//;
-			push @{$this->{'REQUIRES'}},{'name'=>$name,'desc'=>$rem};
+			push @{$self->{'REQUIRES'}},{'name'=>$name,'desc'=>$rem};
 		}
 
 
 		if (($line=~ m/^\s*use base +([^\; ]+)[\;](.*)/) ||
 			($line=~ m/^\s*our +\@ISA +([^\; ]+)[\;](.*)/)){
-			$this->{'INHERITS_FROM'} = $this->{'INHERITS_FROM'} || [];
+			$self->{'INHERITS_FROM'} = $self->{'INHERITS_FROM'} || [];
 			my $name=$1;
 			my $rem=$2;
 			$name=~ s/qw\(//g;
 			$name=~ s/[\)\']//g;
 			my @n=split(/ +/,$name);
 			foreach my $n (@n){
-				push @{$this->{'INHERITS_FROM'}},{'name'=>$n} if $n;	
+				push @{$self->{'INHERITS_FROM'}},{'name'=>$n} if $n;	
 			}
 		}
 		
-		#print $line.'   -   '.$this->{'STATE'};
+		#print $line.'   -   '.$self->{'STATE'};
 	}
 	
 	
-	if ((exists $this->{'METHOD_ATTR'}->{'new'}) || (scalar($this->{'INHERITS_FROM'}) >= 1 )){ ## its a class!
-		$this->{'ISCLASS'}=1;
+	if ((exists $self->{'METHOD_ATTR'}->{'new'}) || (scalar($self->{'INHERITS_FROM'}) >= 1 )){ ## its a class!
+		$self->{'ISCLASS'}=1;
 	}
 	
 	
-#	print Dumper($this->{'METHOD_ATTR'});
-	$this->_analyseAttributes();
+#	print Dumper($self->{'METHOD_ATTR'});
+	$self->_analyseAttributes();
 
 
-	$this->_scanDescription($arr);
+	$self->_scanDescription($arr);
 
 
-	#print Dumper($this->{'METHOD_ATTR'});
+	#print Dumper($self->{'METHOD_ATTR'});
 
 	
 }
@@ -648,10 +664,10 @@ my $arr=shift or die "Arrayref expected";
 
 
 sub _scanDescription{
-my $this=shift;	
+my $self=shift;	
 my $arr=shift or die "Arrayref expected";	
 	
-	$this->{'STATE'} = 'head';
+	$self->{'STATE'} = 'head';
 	
 	my @text;
 	
@@ -660,7 +676,7 @@ my $arr=shift or die "Arrayref expected";
 		
 		my $line = $arr->[$i];
 		
-		if (($line=~ m/^\s*\#+(.*)/) && ($state=~ m/^(wait|rem)$/)){			
+		if (($line=~ m/^\s*\#+(.*)/) && ($state=~ m/^(wait|rem)$/)){	
 			$state='rem';
 			$line=~ m/^\s*\#+(.*)/;
 			my $text=$1;
@@ -672,9 +688,9 @@ my $arr=shift or die "Arrayref expected";
 	}
 	
 	
-	my $more = $this->_findOwnTitlesInArray(array=>\@text, default=>'DESCRIPTION');
+	my $more = $self->_findOwnTitlesInArray(array=>\@text, default=>'DESCRIPTION');
 	
-	$this->{'MORE'} = $more;
+	$self->{'MORE'} = $more;
 
 }
 
@@ -683,13 +699,13 @@ my $arr=shift or die "Arrayref expected";
 
 
 sub _findOwnTitlesInArray{
-my $this=shift;	
+my $self=shift;	
 my $v={@_};
 my $arr=$v->{'array'}  or die "Array expected";
 my $default=$v->{'default'};
 my $morearr={};
 
-	$this->_prepareArrayText(array=>$arr);
+	$self->_prepareArrayText(array=>$arr);
 
 	my $area = $default;
 
@@ -702,7 +718,7 @@ my $morearr={};
 		## is introduction?
 		if ($next=~ m/^\s*(\={3,50})/){ ## find a ==== bar
 			my $l=length($1);
-			$area=$this->_trim($line);
+			$area=$self->_trim($line);
 			$nextok=$i+2; ## skip next 2 rows
 		}
 		
@@ -723,14 +739,14 @@ return $morearr;
 
 
 sub _addLineToHeadBuffer{
-my $this=shift;
+my $self=shift;
 my $line=shift;
 
-	$line = $this->_trim($line);
+	$line = $self->_trim($line);
 
-	$this->{'HEAD'} = $this->{'HEAD'} || [];
+	$self->{'HEAD'} = $self->{'HEAD'} || [];
 	
-	unshift @{$this->{'HEAD'}},$line;
+	unshift @{$self->{'HEAD'}},$line;
 		
 
 }
@@ -739,21 +755,21 @@ my $line=shift;
 
 
 sub _addLineToBodyBuffer{
-my $this=shift;
+my $self=shift;
 my $line=shift;
 
-	$line = $this->_trim($line);
+	$line = $self->_trim($line);
 
 	if ($line=~ m/^\s*return (.*)/){
-		if (!$this->_getMethodReturn){
-			$this->_setMethodReturn($line);	
+		if (!$self->_getMethodReturn){
+			$self->_setMethodReturn($line);	
 		}
 	}
 
 
-	$this->{'BODY'} = $this->{'BODY'} || [];
+	$self->{'BODY'} = $self->{'BODY'} || [];
 	
-	unshift @{$this->{'BODY'}},$line;
+	unshift @{$self->{'BODY'}},$line;
 		
 
 }
@@ -761,12 +777,12 @@ my $line=shift;
 
 
 sub _clearBodyBuffer{
-my $this=shift;
+my $self=shift;
 my $line=shift;
 
-	$line = $this->_trim($line);
+	$line = $self->_trim($line);
 
-	$this->{'BODY'} = [];
+	$self->{'BODY'} = [];
 
 }
 
@@ -774,68 +790,68 @@ my $line=shift;
 
 
 sub _clearHeadBuffer{
-my $this=shift;
+my $self=shift;
 my $line=shift;
 
-	$line = $this->_trim($line);
+	$line = $self->_trim($line);
 
-	$this->{'HEAD'} = [];
+	$self->{'HEAD'} = [];
 
 }
 
 
 sub _addHeadBufferToAttr{
-my $this=shift;
+my $self=shift;
 
-	my $m = $this->_getMethodName();
+	my $m = $self->_getMethodName();
 	if ($m){
-		$this->_setMethodAttr($m,'head',$this->{'HEAD'})
+		$self->_setMethodAttr($m,'head',$self->{'HEAD'})
 	}
 }
 
 
 
 sub _addBodyBufferToAttr{
-my $this=shift;
+my $self=shift;
 
-	my $m = $this->_getMethodName();
-	$this->_setMethodAttr($m,'body',$this->{'BODY'})
+	my $m = $self->_getMethodName();
+	$self->_setMethodAttr($m,'body',$self->{'BODY'})
 }
 
 
 
 
 sub _setMethodLine{
-my $this=shift;
+my $self=shift;
 my $s=shift;
 
-	$s = $this->_trim($s);
+	$s = $self->_trim($s);
 	
 	if ($s=~ m/sub ([^ \{]+)(.*)/){
-		$this->_setMethodName($1);
-		$this->_setMethodAttr($1,'methodlinerest',$2);
+		$self->_setMethodName($1);
+		$self->_setMethodAttr($1,'methodlinerest',$2);
 	}
 
 
-$this->{'METHOD_LINE'}=$s;
+$self->{'METHOD_LINE'}=$s;
 }
 
 
 
 sub _getMethodLine{
-my $this=shift;
+my $self=shift;
 
-return $this->{'METHOD_LINE'};
+return $self->{'METHOD_LINE'};
 }
 
 
 
 sub _setMethodName{
-my $this=shift;
+my $self=shift;
 my $s=shift;
 
 
-$this->{'METHOD_NAME'}=$s;
+$self->{'METHOD_NAME'}=$s;
 }
 
 
@@ -843,19 +859,19 @@ $this->{'METHOD_NAME'}=$s;
 
 
 sub _getMethodReturn{
-my $this=shift;
+my $self=shift;
 
-return $this->{'METHOD_RETURN'};
+return $self->{'METHOD_RETURN'};
 }
 
 
 
 sub _setMethodReturn{
-my $this=shift;
+my $self=shift;
 my $s=shift;
 
 
-$this->{'METHOD_RETURN'}=$s;
+$self->{'METHOD_RETURN'}=$s;
 }
 
 
@@ -863,22 +879,22 @@ $this->{'METHOD_RETURN'}=$s;
 
 
 sub _getMethodName{
-my $this=shift;
+my $self=shift;
 
 
-return $this->{'METHOD_NAME'};
+return $self->{'METHOD_NAME'};
 }
 
 
 
 
 sub _setMethodAttr{
-my $this=shift;
+my $self=shift;
 my $name=shift;
 my $k=shift;
 my $s=shift;
 
-$this->{'METHOD_ATTR'}->{$name}->{$k}=$s;
+$self->{'METHOD_ATTR'}->{$name}->{$k}=$s;
 }
 
 
@@ -886,7 +902,7 @@ $this->{'METHOD_ATTR'}->{$name}->{$k}=$s;
 
 
 sub _trim{
-my $this=shift;
+my $self=shift;
 my $s=shift;
 
 	if (ref $s){
@@ -909,15 +925,15 @@ my $s=shift;
 
 
 sub _analyseAttributes{
-my $this=shift;
-my $attr = $this->{'METHOD_ATTR'};
+my $self=shift;
+my $attr = $self->{'METHOD_ATTR'};
 
 
 	foreach my $method (keys %$attr){
 		my $mat=$attr->{$method};
 		
-		$this->_analyseAttributes_Method(attributes=>$mat,method=>$method);
-		$this->_analyseAttributes_Head(attributes=>$mat,method=>$method);
+		$self->_analyseAttributes_Method(attributes=>$mat,method=>$method);
+		$self->_analyseAttributes_Head(attributes=>$mat,method=>$method);
 	}
 	
 	
@@ -928,7 +944,7 @@ my $attr = $this->{'METHOD_ATTR'};
 
 
 sub _analyseAttributes_Method{
-my $this=shift;
+my $self=shift;
 my $v={@_};
 my $method=$v->{'method'};
 my $mat=$v->{'attributes'};
@@ -942,8 +958,8 @@ my $mat=$v->{'attributes'};
 	$at=~ s/\)//;
 
 
-	$mat->{'returntypes'} = $this->_getTypeTreeByLine($re);
-	$mat->{'attributetypes'} = $this->_getTypeTreeByLine($at);
+	$mat->{'returntypes'} = $self->_getTypeTreeByLine($re);
+	$mat->{'attributetypes'} = $self->_getTypeTreeByLine($at);
 
 	
 }
@@ -956,13 +972,13 @@ my $mat=$v->{'attributes'};
 
 
 sub _analyseAttributes_Head{
-my $this=shift;
+my $self=shift;
 my $v={@_};
 my $method=$v->{'method'};
 my $mat=$v->{'attributes'};
 
 
-	$this->_prepareArrayText(array=>$mat->{'head'});
+	$self->_prepareArrayText(array=>$mat->{'head'});
 
 }
 
@@ -970,7 +986,7 @@ my $mat=$v->{'attributes'};
 
 
 sub _prepareArrayText{
-my $this=shift;
+my $self=shift;
 my $v={@_};
 my $array=$v->{'array'};
 
@@ -1007,7 +1023,7 @@ my $array=$v->{'array'};
 
 
 sub _getTypeTreeByLine{
-my $this=shift;
+my $self=shift;
 my $line=shift;
 
 	
@@ -1015,7 +1031,7 @@ my $line=shift;
 	
 	my @rettype;
 	foreach my $s (@re){
-		$s=$this->_trim($s);
+		$s=$self->_trim($s);
 
 
 		my @or = split(/\|/,$s);
@@ -1072,19 +1088,19 @@ return  \@rettype;
 
 # Builds the pod. Called automatically when imporing a perl code.
 sub buildPod{
-my $this=shift;
-my $attr = $this->{'METHOD_ATTR'};
+my $self=shift;
+my $attr = $self->{'METHOD_ATTR'};
 
-	$this->{'POD_PARTS'}={};
+	$self->{'POD_PARTS'}={};
 
-	$this->_buildPod_Name();
-	$this->_buildPod_Methods();
-	$this->_buildPod_Requires();
-	$this->_buildPod_Inherits();
-	$this->_buildPod_More();
+	$self->_buildPod_Name();
+	$self->_buildPod_Methods();
+	$self->_buildPod_Requires();
+	$self->_buildPod_Inherits();
+	$self->_buildPod_More();
 
 
-	$this->_buildPodText();
+	$self->_buildPodText();
 
 }
 
@@ -1093,9 +1109,9 @@ my $attr = $this->{'METHOD_ATTR'};
 
 
 sub _buildPod_Requires{
-my $this=shift;
+my $self=shift;
 
-	my $re=$this->{'REQUIRES'} || [];
+	my $re=$self->{'REQUIRES'} || [];
 
 
 	my %dontshow;
@@ -1116,14 +1132,14 @@ my $this=shift;
 
 			if (!$dontshow{$name}){
 
-				$desc=$this->_trim($desc);
+				$desc=$self->_trim($desc);
 				my $text = "L<$name> $desc\n\n";
 
 				$node->push( node->text($text));
 			}		
 		}
 		
-		$this->{'POD_PARTS'}->{'REQUIRES'} = $node;	
+		$self->{'POD_PARTS'}->{'REQUIRES'} = $node;	
 	}
 
 }
@@ -1133,9 +1149,9 @@ my $this=shift;
 
 
 sub _buildPod_Inherits{
-my $this=shift;
+my $self=shift;
 
-	my $re=$this->{'INHERITS_FROM'} || [];
+	my $re=$self->{'INHERITS_FROM'} || [];
 
 	my %dontshow;
 	my @dontshow = qw(vars strict warnings libs base);
@@ -1155,14 +1171,14 @@ my $this=shift;
 
 			if (!$dontshow{$name}){
 
-				$desc=$this->_trim($desc);
+				$desc=$self->_trim($desc);
 				my $text = "L<$name> $desc\n\n";
 
 				$node->push( node->text($text));
 			}		
 		}
 		
-		$this->{'POD_PARTS'}->{'IMPLEMENTS'} = $node;	
+		$self->{'POD_PARTS'}->{'IMPLEMENTS'} = $node;	
 	}
 
 }
@@ -1171,9 +1187,9 @@ my $this=shift;
 
 
 sub _buildPodText{
-my $this=shift;
+my $self=shift;
 
-	my $parts=$this->{'POD_PARTS'};
+	my $parts=$self->{'POD_PARTS'};
 
 	my @text;
 
@@ -1211,7 +1227,7 @@ my $this=shift;
 	
 	my $text=join("\n",@text);
 
-	$this->{"POD_TEXT"} = $text;
+	$self->{"POD_TEXT"} = $text;
 }
 
 
@@ -1219,9 +1235,9 @@ my $this=shift;
 
 
 sub _buildPod_Name{
-my $this=shift;
-my $attr = $this->{'METHOD_ATTR'};
-my $name = $this->{'PKGNAME'};
+my $self=shift;
+my $attr = $self->{'METHOD_ATTR'};
+my $name = $self->{'PKGNAME'};
 
 	my $node = node->root;
 
@@ -1229,8 +1245,8 @@ my $name = $this->{'PKGNAME'};
 	
 	my @name;
 	
-	push @name,$this->{'PKGNAME'};
-	push @name,$this->_trim($this->{'PKGNAME_DESC'}) if $this->{'PKGNAME_DESC'};
+	push @name,$self->{'PKGNAME'};
+	push @name,$self->_trim($self->{'PKGNAME_DESC'}) if $self->{'PKGNAME_DESC'};
 	
 	my $namestr = join(" - ",@name)."\n\n";
 	
@@ -1238,7 +1254,7 @@ my $name = $this->{'PKGNAME'};
 
 
 
-	$this->{'POD_PARTS'}->{'NAME'} = $node;
+	$self->{'POD_PARTS'}->{'NAME'} = $node;
 
 }
 
@@ -1249,27 +1265,27 @@ my $name = $this->{'PKGNAME'};
 
 
 sub _buildPod_More{
-my $this=shift;
-my $attr = $this->{'METHOD_ATTR'};
+my $self=shift;
+my $attr = $self->{'METHOD_ATTR'};
 
 
 
-	my $more = $this->{'MORE'};
+	my $more = $self->{'MORE'};
 
 	foreach my $area (keys %$more){
 
 		my $node = node->root;
 			
 		my $desc=$more->{$area};
-		
-		if (length(@$desc) > 0){
+                # length(@$desc) throws an error on newer perl, so use scalar(@$desc) instead. NW		
+		if (scalar(@$desc) > 0){
 	
 			$node->push( node->head1("$area") );
 			$node->push( node->text( join("\n",@$desc)."\n\n" ));
 				
 		}
 
-		$this->{'POD_PARTS'}->{$area} = $node;
+		$self->{'POD_PARTS'}->{$area} = $node;
 	}
 
 
@@ -1281,8 +1297,8 @@ my $attr = $this->{'METHOD_ATTR'};
 
 
 sub _buildPod_Methods{
-my $this=shift;
-my $attr = $this->{'METHOD_ATTR'};
+my $self=shift;
+my $attr = $self->{'METHOD_ATTR'};
 
 	my $node = node->root;
 
@@ -1293,7 +1309,7 @@ my $attr = $this->{'METHOD_ATTR'};
 	@methods = sort @methods;
 
 	if (exists $attr->{'new'}){ ## constructor first 
-		$this->_buildPod_Methods_addMethod(node=>$node,method=>'new');
+		$self->_buildPod_Methods_addMethod(node=>$node,method=>'new');
 	}
 
 	foreach my $method (@methods){
@@ -1302,35 +1318,35 @@ my $attr = $this->{'METHOD_ATTR'};
 
 		if ($method=~ m/^\_/){
 			$ok=0;
-			if ($this->{'alsohiddenmethods'}){$ok=1};
+			if ($self->{'alsohiddenmethods'}){$ok=1};
 		}
 
 		if ($ok){
 			if ($method ne 'new'){
-				$this->_buildPod_Methods_addMethod(node=>$node,method=>$method);
+				$self->_buildPod_Methods_addMethod(node=>$node,method=>$method);
 			}
 		}
 		
 	}
 
 	
-	$this->{'POD_PARTS'}->{'METHODS'} = $node;
+	$self->{'POD_PARTS'}->{'METHODS'} = $node;
 }
 
 
 
 
 sub _buildPod_Methods_addMethod{
-my $this=shift;
+my $self=shift;
 my $v={@_};
 my $node=$v->{'node'};
 my $method=$v->{'method'};
-my $attr = $this->{'METHOD_ATTR'};
+my $attr = $self->{'METHOD_ATTR'};
 my $mat=$attr->{$method};
 
 	my $selfstring='';
-	if ($this->{'ISCLASS'}){
-		$selfstring=$this->{'selfstring'}.'->';	
+	if ($self->{'ISCLASS'}){
+		$selfstring=$self->{'selfstring'}.'->';	
 	}
 	
 
@@ -1340,8 +1356,8 @@ my $mat=$attr->{$method};
 
 	## how to call
 
-	my $retstring = $this->_buildParamString(params=>$mat->{'returntypes'}, braces=>1,separatorand=>', ',separatoror=>' | ');
-	my $paramstring = $this->_buildParamString(params=>$mat->{'attributetypes'}, braces=>0,separatorand=>', ',separatoror=>' | ');
+	my $retstring = $self->_buildParamString(params=>$mat->{'returntypes'}, braces=>1,separatorand=>', ',separatoror=>' | ');
+	my $paramstring = $self->_buildParamString(params=>$mat->{'attributetypes'}, braces=>0,separatorand=>', ',separatoror=>' | ');
 
 	my $addit=0;
 	if ($retstring){
@@ -1366,7 +1382,7 @@ my $mat=$attr->{$method};
 
 	my $text;
 	if ($mat->{'head'}){
-		$text = join("",@{ $mat->{'head'} });
+		$text = join("\n",@{ $mat->{'head'} }); ## I added the return here, which is necessary using example codes before methods
 		if ($text){$text.="\n\n\n"};
 	
 		$node->push( node->text($text) );
@@ -1379,7 +1395,7 @@ my $mat=$attr->{$method};
 
 
 sub _buildParamString{
-my $this=shift;
+my $self=shift;
 my $v={@_};
 my $params=$v->{'params'};
 my $braces=$v->{'braces'};
@@ -1402,11 +1418,11 @@ my $text='';
 			my $wname = $name || $type;
 	
 			if ($type ne 'keyvalue'){
-				my $ctype=$this->_typeToChar($type);
+				my $ctype=$self->_typeToChar($type);
 				push @or,"$ctype$wname";
 			}else{
 				my $typev = $e->{'typevalue'};
-				my $ctype=$this->_typeToChar($typev);
+				my $ctype=$self->_typeToChar($typev);
 				push @or,"$name => $ctype$typev";
 			}
 			
@@ -1427,7 +1443,7 @@ return $text;
 
 
 sub _typeToChar{
-my $this=shift;
+my $self=shift;
 my $type=shift;
 my $c='';
 
@@ -1448,6 +1464,48 @@ return $c;
 
 
 
+
+sub _makeDirRecursive{
+my $dir=shift;
+my $path;
+
+  if (!-e $dir){
+
+    my @path=split(/\//,$dir);
+
+    foreach my $p (@path){
+      if (!-e $p){
+        mkdir $path.$p
+        #print $path.$p."\n";
+      }
+      $path.=$p.'/';
+    }
+
+  }
+}
+
+
+
+sub _extractPath{
+my $p=shift;
+
+  if ($p=~ m/\//){
+    $p=~ s/(.*)\/(.*)$/$1/;
+  }else{
+    if ($p=~ m/^\.*$/){ # only ".."
+      $p=$p; ## nothing to do
+    }else{
+      $p='';
+    }
+  }
+
+return $p;
+}
+
+
+
+
+
 1;
 
 
@@ -1523,7 +1581,7 @@ before the sub{} definition:
 
  # This method is doing foo.
  #
- #  print $this->foo();
+ #  print $self->foo();
  #
  # 
  # It is not doing bar, only foo.
@@ -1545,9 +1603,9 @@ way is necessary, but it is designed to type it rapidly.
 
 The example above produces the following method description: 
 
- $this->foo($text);
+ $self->foo($text);
 
-The object "$this" is the default and automatially used when a constructor was found ("new")
+The object "$self" is the default and automatially used when a constructor was found ("new")
 or the class inherits with ISA or "use base".
 You can change this by the parameter "selfstring" in the autopod constructor.
 
@@ -1565,7 +1623,7 @@ That procudes the same output! It means the dollar sign of the first example is
 
 Produces:
 
- $this->foo($scalar);
+ $self->foo($scalar);
 
 As you see, that was the quickest way to write the definition. The keywork "void" is default.
 
@@ -1588,7 +1646,7 @@ Now a more complex example:
 
 produces:
 
- my $state = $this->foo($firstname, $lastname, \%persondata);
+ my $state = $self->foo($firstname, $lastname, \%persondata);
 
 or write it in java style:
 
@@ -1604,7 +1662,7 @@ Multiple return values may be displayed as following:
 
 produces:
 
- my ($a, $b) = $this->foo($text);
+ my ($a, $b) = $self->foo($text);
 
 
 If you want to use key values pairs as in a hash, you may describe it like:
@@ -1623,7 +1681,7 @@ There is also a way to expain that a value A OR B is expected. See here:
 
 procudes:
 
-  my $lista | \$refb = $this->foo(\@list | $text, $flag);
+  my $lista | \$refb = $self->foo(\@list | $text, $flag);
 
 Of course, that is not an official perl syntax with the or "|", but it shows
 you that is expected.
@@ -1647,84 +1705,77 @@ example LICENSE is allways near the end.
 
 =head2 new
 
- my $object = $this->new($filename => $scalar, alsohiddenmethods => $scalar, selfstring => $scalar);
+ my $object = $self->new($filename => $scalar, alsohiddenmethods => $scalar, selfstring => $scalar);
 
-ConstructorThe keyvalues are not mandatory.selfstring may hold something like '$self' as alternative to '$this', which is default.alsohiddenmethods gets a boolean flag to show also methods which starts with "_".
+ConstructorThe keyvalues are not mandatory.selfstring may hold something like '$self' as alternative to '$self', which is default.alsohiddenmethods gets a boolean flag to show also methods which starts with "_".
 
 
 =head2 buildPod
 
- $this->buildPod();
+ $self->buildPod();
 
 Builds the pod. Called automatically when imporing a perl code.
 
 
-=head2 foo
-
- $this->foo();
-
-This method is doing foo. print $this->foo();It is not doing bar, only foo.
-
-
 =head2 getBorderString
 
- my $scalar = $this->getBorderString();
+ my $scalar = $self->getBorderString();
 
 Returns the border string which delimit the perl code and pod inside a pm file.
 
 
 =head2 getPerlCode
 
- my $text = $this->getPerlCode();
+ my $text = $self->getPerlCode();
 
 Returns perl code which was set before.
 
 
 =head2 getPod
 
- my $text = $this->getPod();
+ my $text = $self->getPod();
 
 Returns the pod formated text.s
 
 
 =head2 readDirectory
 
- $this->readDirectory($directory, updateonly => $scalar, pod => $scalar, verbose => $scalar);
+ $self->readDirectory($directory, updateonly => $scalar, pod => $scalar, verbose => $scalar);
 
 scans a directoy recoursively for pm files and maygenerate pod of them.You can also set the flag updateonly to build new podonly for files you already build a pod (inside the file)in the past. Alternatively you can write the magic wordAUTOPODME somewhere in the pm file what signals that thispm file wants to be pod'ed by autopod.The flag pod let will build a separate file. If poddir set,the generated pod file will be saved to a deparate directory.With verbose it prints the list of written files.
 
 
 =head2 readFile
 
- $this->readFile($filename);
+ $self->readFile($filename);
 
 Reading a Perl class file and loads it to memory.
 
 
 =head2 scanArray
 
- $this->scanArray();
+ $self->scanArray();
 
 This class may scan the perl code.But it is called automatically when importing a perl code.
 
 
 =head2 setBorderString
 
- $this->setBorderString($borderstring);
+ $self->setBorderString($borderstring);
 
 Set an alternative border string.If you change this, you have to do it again when updating the pod.
 
 
 =head2 setPerlCode
 
- $this->setPerlCode($text | \@array);
+ $self->setPerlCode($text | \@array);
 
 Expects Perl code as arrayrefor text (scalar).When used, it automatically runs scanArray().
 
 
 =head2 writeFile
 
- $this->writeFile($filename);
+ $self->writeFile($filename);
 
 writes a pod fileIf the file has a pm extension, it writes the perl code and the podIf the file has a pod extension or any, it only writes the pod.
 
